@@ -1,8 +1,6 @@
 /*
  *  Hamlib CI-V backend - main file
- *  Copyright (c) 2000-2009 by Stephane Fillod
- *
- *	$Id: icom.c,v 1.107 2008-11-13 20:29:43 y32kn Exp $
+ *  Copyright (c) 2000-2010 by Stephane Fillod
  *
  *   This library is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU Library General Public License as
@@ -268,6 +266,7 @@ static const struct icom_addr icom_addr_list[] = {
 	{ RIG_MODEL_IC756PRO, 0x5c },
 	{ RIG_MODEL_IC756PROII, 0x64 },
 	{ RIG_MODEL_IC756PROIII, 0x6e },
+	{ RIG_MODEL_IC7600, 0x7a },
 	{ RIG_MODEL_IC761, 0x1e },
 	{ RIG_MODEL_IC765, 0x2c },
 	{ RIG_MODEL_IC775, 0x46 },
@@ -792,9 +791,6 @@ int icom_set_level(RIG *rig, vfo_t vfo, setting_t level, value_t val)
 	lvl_len = 2;
 	to_bcd_be(lvlbuf, (long long)icom_val, lvl_len*2);
 
-	/* Optimize:
-	 *   sort the switch cases with the most frequent first
-	 */
 	switch (level) {
 	case RIG_LEVEL_PREAMP:
 		lvl_cn = C_CTL_FUNC;
@@ -888,10 +884,16 @@ int icom_set_level(RIG *rig, vfo_t vfo, setting_t level, value_t val)
 	case RIG_LEVEL_AGC:
 		lvl_cn = C_CTL_FUNC;
 		lvl_sc = S_FUNC_AGC;
-		if (rig->caps->rig_model == RIG_MODEL_ICR75) {
-			lvl_len = 1;
-			lvlbuf[0] = val.i;
-		}
+		lvl_len = 1;
+        switch (val.i) {
+            case RIG_AGC_SLOW:   lvlbuf[0] = D_AGC_SLOW; break;
+            case RIG_AGC_MEDIUM: lvlbuf[0] = D_AGC_MID; break;
+            case RIG_AGC_FAST:   lvlbuf[0] = D_AGC_FAST; break;
+            case RIG_AGC_SUPERFAST: lvlbuf[0] = D_AGC_SUPERFAST; break;
+            default:
+                rig_debug(RIG_DEBUG_ERR,"Unsupported LEVEL_AGC %d", val.i);
+                return -RIG_EINVAL;
+        }
 		break;
 	case RIG_LEVEL_BKINDL:
 		lvl_cn = C_CTL_LVL;
@@ -901,17 +903,29 @@ int icom_set_level(RIG *rig, vfo_t vfo, setting_t level, value_t val)
 		lvl_cn = C_CTL_LVL;
 		lvl_sc = S_LVL_BALANCE;
 		break;
-        case RIG_LEVEL_VOXGAIN:     /* IC-910H */
-            lvl_cn = C_CTL_MEM;
-            lvl_sc = S_MEM_VOXGAIN;
+        case RIG_LEVEL_VOXGAIN:
+            if (rig->caps->rig_model == RIG_MODEL_IC910) {
+                /* IC-910H */
+                lvl_cn = C_CTL_MEM;
+                lvl_sc = S_MEM_VOXGAIN;
+            } else {
+                lvl_cn = C_CTL_LVL;
+                lvl_sc = S_LVL_VOXGAIN;
+            }
             break;
         case RIG_LEVEL_VOXDELAY:    /* IC-910H */
             lvl_cn = C_CTL_MEM;
             lvl_sc = S_MEM_VOXDELAY;
             break;
-        case RIG_LEVEL_ANTIVOX:     /* IC-910H */
-            lvl_cn = C_CTL_MEM;
-            lvl_sc = S_MEM_ANTIVOX;
+        case RIG_LEVEL_ANTIVOX:
+            if (rig->caps->rig_model == RIG_MODEL_IC910) {
+                /* IC-910H */
+                lvl_cn = C_CTL_MEM;
+                lvl_sc = S_MEM_ANTIVOX;
+            } else {
+                lvl_cn = C_CTL_LVL;
+                lvl_sc = S_LVL_ANTIVOX;
+            }
             break;
 	default:
 		rig_debug(RIG_DEBUG_ERR,"Unsupported set_level %d", level);
@@ -935,6 +949,12 @@ int icom_set_level(RIG *rig, vfo_t vfo, setting_t level, value_t val)
 /*
  * icom_get_level
  * Assumes rig!=NULL, rig->state.priv!=NULL, val!=NULL
+ *
+ * TODO (missing RIG_LEVEL):
+ * - S_RFML: Read real RFpower-meter level
+ * - S_CMP: Read COMP-meter level
+ * - S_VD : Read Vd-meter level
+ * - S_ID : Read Id-meter level
  */
 int icom_get_level(RIG *rig, vfo_t vfo, setting_t level, value_t *val)
 {
@@ -951,15 +971,21 @@ int icom_get_level(RIG *rig, vfo_t vfo, setting_t level, value_t *val)
 	priv = (struct icom_priv_data*)rs->priv;
 
 	lvl2_len = 0;
-	/* Optimize:
-	 *   sort the switch cases with the most frequent first
-	 */
-	switch (level) {
+
+    switch (level) {
 	case RIG_LEVEL_STRENGTH:
 	case RIG_LEVEL_RAWSTR:
 		lvl_cn = C_RD_SQSM;
 		lvl_sc = S_SML;
 		break;
+	case RIG_LEVEL_ALC:
+		lvl_cn = C_RD_SQSM;
+		lvl_sc = S_ALC;
+		break;
+	case RIG_LEVEL_SWR:
+		lvl_cn = C_RD_SQSM;
+		lvl_sc = S_SWR;
+		break;
 	case RIG_LEVEL_PREAMP:
 		lvl_cn = C_CTL_FUNC;
 		lvl_sc = S_FUNC_PAMP;
@@ -1044,16 +1070,28 @@ int icom_get_level(RIG *rig, vfo_t vfo, setting_t level, value_t *val)
 		lvl_sc = S_LVL_BALANCE;
 		break;
         case RIG_LEVEL_VOXGAIN:     /* IC-910H */
-            lvl_cn = C_CTL_MEM;
-            lvl_sc = S_MEM_VOXGAIN;
+            if (rig->caps->rig_model == RIG_MODEL_IC910) {
+                /* IC-910H */
+                lvl_cn = C_CTL_MEM;
+                lvl_sc = S_MEM_VOXGAIN;
+            } else {
+                lvl_cn = C_CTL_LVL;
+                lvl_sc = S_LVL_VOXGAIN;
+            }
             break;
         case RIG_LEVEL_VOXDELAY:    /* IC-910H */
             lvl_cn = C_CTL_MEM;
             lvl_sc = S_MEM_VOXDELAY;
             break;
-        case RIG_LEVEL_ANTIVOX:     /* IC-910H */
-            lvl_cn = C_CTL_MEM;
-            lvl_sc = S_MEM_ANTIVOX;
+        case RIG_LEVEL_ANTIVOX:
+            if (rig->caps->rig_model == RIG_MODEL_IC910) {
+                /* IC-910H */
+                lvl_cn = C_CTL_MEM;
+                lvl_sc = S_MEM_ANTIVOX;
+            } else {
+                lvl_cn = C_CTL_LVL;
+                lvl_sc = S_LVL_ANTIVOX;
+            }
             break;
 	default:
 		rig_debug(RIG_DEBUG_ERR,"Unsupported get_level %d", level);
@@ -1091,8 +1129,28 @@ int icom_get_level(RIG *rig, vfo_t vfo, setting_t level, value_t *val)
 
 	switch (level) {
 	case RIG_LEVEL_RAWSTR:
+		/* raw value */
 		val->i = icom_val;
 		break;
+	case RIG_LEVEL_AGC:
+        switch (icom_val) {
+            case D_AGC_SLOW: val->i = RIG_AGC_SLOW; break;
+            case D_AGC_MID:  val->i = RIG_AGC_MEDIUM; break;
+            case D_AGC_FAST: val->i = RIG_AGC_FAST; break;
+            case D_AGC_SUPERFAST: val->i = RIG_AGC_SUPERFAST; break;
+            default:
+                rig_debug(RIG_DEBUG_ERR,"Unexpected AGC 0x%02x", icom_val);
+                return -RIG_EPROTO;
+        }
+		break;
+	case RIG_LEVEL_ALC:
+        /* 120 max on IC-7600 */
+		val->f = (float)icom_val/120;
+		break;
+	case RIG_LEVEL_SWR:
+        /* {0->1, 48->1.5, 80->2} on IC-7600 */
+		val->f = 1. + (float)icom_val/80;
+		break;
 	case RIG_LEVEL_PREAMP:
 		if (icom_val == 0) {
 			val->i = 0;
@@ -1423,6 +1481,22 @@ int icom_get_rptr_offs(RIG *rig, vfo_t vfo, shortfreq_t *rptr_offs)
 	return RIG_OK;
 }
 
+/*
+ * Helper function to go back and forth split VFO
+ */
+static int get_split_vfos(const RIG *rig, vfo_t *rx_vfo, vfo_t *tx_vfo)
+{
+    if ((rig->state.vfo_list & (RIG_VFO_A|RIG_VFO_B)) == (RIG_VFO_A|RIG_VFO_B)) {
+        *rx_vfo = RIG_VFO_A;
+        *tx_vfo = RIG_VFO_B;
+    } else if ((rig->state.vfo_list & (RIG_VFO_MAIN|RIG_VFO_SUB)) == (RIG_VFO_MAIN|RIG_VFO_SUB)) {
+        *rx_vfo = RIG_VFO_MAIN;
+        *tx_vfo = RIG_VFO_SUB;
+    } else {
+        return -RIG_ENAVAIL;
+    }
+    return RIG_OK;
+}
 
 /*
  * icom_set_split_freq
@@ -1435,6 +1509,7 @@ int icom_get_rptr_offs(RIG *rig, vfo_t vfo, shortfreq_t *rptr_offs)
 int icom_set_split_freq(RIG *rig, vfo_t vfo, freq_t tx_freq)
 {
 	int status;
+	vfo_t rx_vfo, tx_vfo;
 
 	/* This method works also in memory mode(RIG_VFO_MEM) */
 	if (rig_has_vfo_op(rig, RIG_OP_XCHG)) {
@@ -1453,7 +1528,11 @@ int icom_set_split_freq(RIG *rig, vfo_t vfo, freq_t tx_freq)
 		return 0;
 	}
 
-	status = icom_set_vfo(rig, RIG_VFO_B);
+	status = get_split_vfos(rig, &rx_vfo, &tx_vfo);
+	if (status != RIG_OK)
+		return status;
+
+	status = icom_set_vfo(rig, tx_vfo);
 	if (status != RIG_OK)
 		return status;
 
@@ -1461,7 +1540,7 @@ int icom_set_split_freq(RIG *rig, vfo_t vfo, freq_t tx_freq)
 	if (status != RIG_OK)
 		return status;
 
-	status = icom_set_vfo(rig, RIG_VFO_A);
+	status = icom_set_vfo(rig, rx_vfo);
 	if (status != RIG_OK)
 		return status;
 
@@ -1477,6 +1556,7 @@ int icom_set_split_freq(RIG *rig, vfo_t vfo, freq_t tx_freq)
 int icom_get_split_freq(RIG *rig, vfo_t vfo, freq_t *tx_freq)
 {
 	int status;
+	vfo_t rx_vfo, tx_vfo;
 
 	/* This method works also in memory mode(RIG_VFO_MEM) */
 	if (rig_has_vfo_op(rig, RIG_OP_XCHG)) {
@@ -1495,7 +1575,11 @@ int icom_get_split_freq(RIG *rig, vfo_t vfo, freq_t *tx_freq)
 		return 0;
 	}
 
-	status = icom_set_vfo(rig, RIG_VFO_B);
+	status = get_split_vfos(rig, &rx_vfo, &tx_vfo);
+	if (status != RIG_OK)
+		return status;
+
+	status = icom_set_vfo(rig, tx_vfo);
 	if (status != RIG_OK)
 		return status;
 
@@ -1503,7 +1587,7 @@ int icom_get_split_freq(RIG *rig, vfo_t vfo, freq_t *tx_freq)
 	if (status != RIG_OK)
 		return status;
 
-	status = icom_set_vfo(rig, RIG_VFO_A);
+	status = icom_set_vfo(rig, rx_vfo);
 	if (status != RIG_OK)
 		return status;
 
@@ -1519,6 +1603,7 @@ int icom_get_split_freq(RIG *rig, vfo_t vfo, freq_t *tx_freq)
 int icom_set_split_mode(RIG *rig, vfo_t vfo, rmode_t tx_mode, pbwidth_t tx_width)
 {
 	int status;
+	vfo_t rx_vfo, tx_vfo;
 
 	/* This method works also in memory mode(RIG_VFO_MEM) */
 	if (rig_has_vfo_op(rig, RIG_OP_XCHG)) {
@@ -1537,7 +1622,11 @@ int icom_set_split_mode(RIG *rig, vfo_t vfo, rmode_t tx_mode, pbwidth_t tx_width
 		return 0;
 	}
 
-	status = icom_set_vfo(rig, RIG_VFO_B);
+	status = get_split_vfos(rig, &rx_vfo, &tx_vfo);
+	if (status != RIG_OK)
+		return status;
+
+	status = icom_set_vfo(rig, tx_vfo);
 	if (status != RIG_OK)
 		return status;
 
@@ -1545,7 +1634,7 @@ int icom_set_split_mode(RIG *rig, vfo_t vfo, rmode_t tx_mode, pbwidth_t tx_width
 	if (status != RIG_OK)
 		return status;
 
-	status = icom_set_vfo(rig, RIG_VFO_A);
+	status = icom_set_vfo(rig, rx_vfo);
 	if (status != RIG_OK)
 		return status;
 
@@ -1562,6 +1651,7 @@ int icom_set_split_mode(RIG *rig, vfo_t vfo, rmode_t tx_mode, pbwidth_t tx_width
 int icom_get_split_mode(RIG *rig, vfo_t vfo, rmode_t *tx_mode, pbwidth_t *tx_width)
 {
 	int status;
+	vfo_t rx_vfo, tx_vfo;
 
 	/* This method works also in memory mode(RIG_VFO_MEM) */
 	if (rig_has_vfo_op(rig, RIG_OP_XCHG)) {
@@ -1580,7 +1670,11 @@ int icom_get_split_mode(RIG *rig, vfo_t vfo, rmode_t *tx_mode, pbwidth_t *tx_wid
 		return 0;
 	}
 
-	status = icom_set_vfo(rig, RIG_VFO_B);
+	status = get_split_vfos(rig, &rx_vfo, &tx_vfo);
+	if (status != RIG_OK)
+		return status;
+
+	status = icom_set_vfo(rig, tx_vfo);
 	if (status != RIG_OK)
 		return status;
 
@@ -1588,7 +1682,7 @@ int icom_get_split_mode(RIG *rig, vfo_t vfo, rmode_t *tx_mode, pbwidth_t *tx_wid
 	if (status != RIG_OK)
 		return status;
 
-	status = icom_set_vfo(rig, RIG_VFO_A);
+	status = icom_set_vfo(rig, rx_vfo);
 	if (status != RIG_OK)
 		return status;
 
@@ -1687,7 +1781,8 @@ int icom_mem_get_split_vfo(RIG *rig, vfo_t vfo, split_t *split, vfo_t *tx_vfo)
 	/* this hacks works only when in memory mode
 	 * I have no clue how to detect split in regular VFO mode
 	 */
-	if (rig->state.current_vfo != RIG_VFO_MEM)
+	if (rig->state.current_vfo != RIG_VFO_MEM ||
+            !rig_has_vfo_op(rig, RIG_OP_XCHG))
 		return -RIG_ENAVAIL;
 
 	retval = icom_vfo_op(rig, vfo, RIG_OP_XCHG);
@@ -1797,9 +1892,6 @@ int icom_set_func(RIG *rig, vfo_t vfo, setting_t func, int status)
 	fctbuf[0] = status? 0x01:0x00;
 	fct_len = rig->caps->rig_model == RIG_MODEL_ICR8500 ? 0 : 1;
 
-	/* Optimize:
-	 *   sort the switch cases with the most frequent first
-	 */
 	switch (func) {
 	case RIG_FUNC_FAGC:
 		fct_cn = C_CTL_FUNC;
@@ -1874,6 +1966,10 @@ int icom_set_func(RIG *rig, vfo_t vfo, setting_t func, int status)
 		fct_cn = C_CTL_FUNC;
 		fct_sc = S_FUNC_VSC;
 		break;
+	case RIG_FUNC_LOCK:
+		fct_cn = C_CTL_FUNC;
+		fct_sc = S_FUNC_DIAL_LK;
+		break;
         case RIG_FUNC_AFC:      /* IC-910H */
             fct_cn = C_CTL_FUNC;
             fct_sc = S_FUNC_AFC;
@@ -1921,9 +2017,6 @@ int icom_get_func(RIG *rig, vfo_t vfo, setting_t func, int *status)
 	int ack_len=sizeof(ackbuf), retval;
 	int fct_cn, fct_sc;		/* Command Number, Subcommand */
 
-	/* Optimize:
-	 *   sort the switch cases with the most frequent first
-	 */
 	switch (func) {
 	case RIG_FUNC_FAGC:
 		fct_cn = C_CTL_FUNC;
@@ -1982,6 +2075,10 @@ int icom_get_func(RIG *rig, vfo_t vfo, setting_t func, int *status)
             fct_cn = C_CTL_FUNC;
             fct_sc = S_FUNC_VSC;
             break;
+	case RIG_FUNC_LOCK:
+		fct_cn = C_CTL_FUNC;
+		fct_sc = S_FUNC_DIAL_LK;
+		break;
          case RIG_FUNC_AFC:      /* IC-910H */
             fct_cn = C_CTL_FUNC;
             fct_sc = S_FUNC_AFC;
@@ -2630,13 +2727,20 @@ int icom_vfo_op(RIG *rig, vfo_t vfo, vfo_op_t op)
 	unsigned char ackbuf[MAXFRAMELEN];
 	int mv_len, ack_len=sizeof(ackbuf), retval;
 	int mv_cn, mv_sc;
+	int vfo_list;
 
 	mv_len = 0;
 
 	switch(op) {
 		case RIG_OP_CPY:
 			mv_cn = C_SET_VFO;
-			mv_sc = S_BTOA;
+            vfo_list = rig->state.vfo_list;
+            if ((vfo_list & (RIG_VFO_A|RIG_VFO_B)) == (RIG_VFO_A|RIG_VFO_B))
+			    mv_sc = S_BTOA;
+            else if ((vfo_list & (RIG_VFO_MAIN|RIG_VFO_SUB)) == (RIG_VFO_MAIN|RIG_VFO_SUB))
+			    mv_sc = S_SUBTOMAIN;
+            else
+                return -RIG_ENAVAIL;
 			break;
 		case RIG_OP_XCHG:
 			mv_cn = C_SET_VFO;
@@ -2715,7 +2819,14 @@ int icom_scan(RIG *rig, vfo_t vfo, scan_t scan, int ch)
 			retval = icom_set_vfo(rig, RIG_VFO_MEM);
 			if (retval != RIG_OK)
 				return retval;
-			scan_sc = S_SCAN_START;
+			/* Looks like all the IC-R* have this command,
+			 * but some old models don't have it.
+			 * Should be put in icom_priv_caps ?
+			 */
+			if (rig->caps->rig_type == RIG_TYPE_RECEIVER)
+			    scan_sc = S_SCAN_MEM2;
+			else
+			    scan_sc = S_SCAN_START;
 			break;
 		case RIG_SCAN_SLCT:
 			retval = icom_set_vfo(rig, RIG_VFO_MEM);
@@ -3019,6 +3130,7 @@ DECLARE_INITRIG_BACKEND(icom)
 	rig_register(&ic735_caps);
 	rig_register(&ic736_caps);
 	rig_register(&ic737_caps);
+	rig_register(&ic738_caps);
 	rig_register(&ic746_caps);
 	rig_register(&ic746pro_caps);
 	rig_register(&ic751_caps);
@@ -3028,7 +3140,9 @@ DECLARE_INITRIG_BACKEND(icom)
 	rig_register(&ic756pro_caps);
 	rig_register(&ic756pro2_caps);
 	rig_register(&ic756pro3_caps);
+	rig_register(&ic7600_caps);
 	rig_register(&ic765_caps);
+	rig_register(&ic7700_caps);
 	rig_register(&ic78_caps);
 	rig_register(&ic7800_caps);
 	rig_register(&ic7000_caps);
@@ -3061,6 +3175,7 @@ DECLARE_INITRIG_BACKEND(icom)
 	rig_register(&os456_caps);
 
 	rig_register(&omnivip_caps);
+	rig_register(&delta2_caps);
 
 	rig_register(&id1_caps);
 
