@@ -36,16 +36,16 @@
 #define THF7_HIGH_MODES (RIG_MODE_FM|RIG_MODE_AM|RIG_MODE_WFM)
 #define THF7_ALL_MODES (THF7_HIGH_MODES|RIG_MODE_SSB|RIG_MODE_CW)
 
-#define THF7_FUNC_ALL (RIG_FUNC_TONE|RIG_FUNC_TSQL|RIG_FUNC_TBURST| \
-		RIG_FUNC_ARO|RIG_FUNC_LOCK|RIG_FUNC_BC)
+#define THF7_FUNC_ALL (RIG_FUNC_ARO|RIG_FUNC_LOCK|RIG_FUNC_BC)
 
 
 /*
  * How increadible, there's no RIG_LEVEL_STRENGTH!
  */
-#define THF7_LEVEL_ALL (RIG_LEVEL_SQL|RIG_LEVEL_RFPOWER|RIG_LEVEL_ATT|RIG_LEVEL_BALANCE)
+#define THF7_LEVEL_ALL (RIG_LEVEL_SQL|RIG_LEVEL_RFPOWER|RIG_LEVEL_ATT|\
+        RIG_LEVEL_BALANCE|RIG_LEVEL_VOXGAIN|RIG_LEVEL_VOXDELAY)
 
-#define THF7_PARMS (RIG_PARM_APO|RIG_PARM_BEEP|RIG_PARM_BACKLIGHT|RIG_PARM_KEYLIGHT)
+#define THF7_PARMS (RIG_PARM_APO|RIG_PARM_BEEP|RIG_PARM_BACKLIGHT)
 
 #define THF7_VFO_OP (RIG_OP_UP|RIG_OP_DOWN)
 
@@ -73,7 +73,6 @@ static const tone_t thf7_ctcss_list[] = {
   1862, 1928, 2035, 2065, 2107, 2181, 2257, 2291, 2336, 2418,
   2503, 2541,
   0
-
 };
 
 static rmode_t thf7_mode_table[KENWOOD_MODE_TABLE_MAX] = {
@@ -98,13 +97,16 @@ static struct kenwood_priv_caps  thf7_priv_caps  = {
 
 static int thf7e_init(RIG *rig);
 static int thf7e_open(RIG *rig);
-static int thf7_set_vfo (RIG *rig, vfo_t vfo);
 static int thf7_get_vfo (RIG *rig, vfo_t *vfo);
 
 /*
  * TH-F7E rig capabilities.
  *
- * Manual: http://www.k6may.com/KenwoodTHF6Tip1.shtml
+ * Manual, thanks to K6MAY: http://www.k6may.com/KenwoodTHF6Tip1.shtml
+ *
+ * TODO:
+ * - set/get_ctcss_tone/sql through set/get_channel() and VR/VW
+ * - emulate RIG_FUNC_TONE|RIG_FUNC_TSQL by setting ctcss_tone/sql to 0/non zero?
  */
 const struct rig_caps thf7e_caps = {
 .rig_model =  RIG_MODEL_THF7E,
@@ -129,14 +131,14 @@ const struct rig_caps thf7e_caps = {
 .retry =  3,
 
 .has_get_func =  THF7_FUNC_ALL,
-.has_set_func =  THF7_FUNC_ALL,
+.has_set_func =  THF7_FUNC_ALL|RIG_FUNC_TBURST,
 .has_get_level =  THF7_LEVEL_ALL,
 .has_set_level =  RIG_LEVEL_SET(THF7_LEVEL_ALL),
 .has_get_parm =  THF7_PARMS,
 .has_set_parm =  THF7_PARMS,
 .level_gran = {
         [LVL_SQL] = { .min = { .i = 0 }, .max = { .i = 5 } },
-        [LVL_RFPOWER] = { .min = { .i = 3 }, .max = { .i = 0 } },
+        [LVL_RFPOWER] = { .min = { .i = 2 }, .max = { .i = 0 } },
 },
 .parm_gran =  {},
 .ctcss_list =  thf7_ctcss_list,
@@ -182,9 +184,23 @@ const struct rig_caps thf7e_caps = {
 	RIG_FRNG_END
   },
 
-	/* region 2 is TH-F6A */
-.rx_range_list2 =  { RIG_FRNG_END, },
-.tx_range_list2 =  { RIG_FRNG_END, },
+	/* region 2 is model TH-F6A in fact */
+.rx_range_list2 = {
+	/* RIG_ANT_2 is internal bar antenna */
+	{MHz(144),MHz(148),THF7_MODES_TX,-1,-1,RIG_VFO_A,RIG_ANT_1},
+	{MHz(220),MHz(225),THF7_MODES_TX,-1,-1,RIG_VFO_A,RIG_ANT_1},
+	{kHz(430),MHz(440),THF7_MODES_TX,-1,-1,RIG_VFO_A,RIG_ANT_1},
+	{kHz(100),MHz(470),THF7_ALL_MODES,-1,-1,RIG_VFO_B, RIG_ANT_1|RIG_ANT_2},
+	{MHz(470),GHz(1.3),THF7_HIGH_MODES,-1,-1,RIG_VFO_B,RIG_ANT_1},
+	RIG_FRNG_END
+  },
+.tx_range_list2 = {
+	/* power actually depends on DC power supply */
+	{MHz(144),MHz(148),THF7_MODES_TX,W(0.05),W(5),RIG_VFO_A,RIG_ANT_1},
+	{MHz(220),MHz(225),THF7_MODES_TX,W(0.05),W(5),RIG_VFO_A,RIG_ANT_1},
+	{MHz(430),MHz(440),THF7_MODES_TX,W(0.05),W(5),RIG_VFO_A,RIG_ANT_1},
+	RIG_FRNG_END
+  },
 
 .tuning_steps =  {
 	/* This table is ordered according to protocol, from '0' to 'b' */
@@ -214,45 +230,44 @@ const struct rig_caps thf7e_caps = {
 		{RIG_MODE_SSB|RIG_MODE_CW, kHz(3)},
 		RIG_FLT_END,
 	},
-.priv =  (void *)&thf7_priv_caps,
 
-.rig_init =  thf7e_init,
-.rig_cleanup = kenwood_cleanup,
-.rig_open =  thf7e_open,
+	.priv		=  (void *)&thf7_priv_caps,
 
-.set_freq =  th_set_freq,
-.get_freq =  th_get_freq,
-.set_mode =  th_set_mode,
-.get_mode =  th_get_mode,
-.set_vfo =  thf7_set_vfo,
-.get_vfo =  thf7_get_vfo,
-.set_ctcss_tone =  th_set_ctcss_tone,
-.get_ctcss_tone =  th_get_ctcss_tone,
-.set_ptt =  kenwood_set_ptt,
-.get_dcd =  kenwood_get_dcd,
-.vfo_op =  kenwood_vfo_op,
-.set_mem =  th_set_mem,
-.get_mem =  th_get_mem,
+	.rig_init	= thf7e_init,
+	.rig_cleanup	= kenwood_cleanup,
+	.rig_open	= thf7e_open,
 
-.set_trn =  th_set_trn,	/* TBC */
-.get_trn =  th_get_trn, /* TBC */
-.decode_event =  th_decode_event,	/* TBC */
+	.set_freq	= th_set_freq,
+	.get_freq	= th_get_freq,
+	.set_mode	= th_set_mode,
+	.get_mode	= th_get_mode,
+	.set_vfo	= th_set_vfo,
+	.get_vfo	= thf7_get_vfo,
+	.set_ptt	= th_set_ptt,
+	.get_dcd	= th_get_dcd,
+	.vfo_op		= kenwood_vfo_op,
+	.set_mem	= th_set_mem,
+	.get_mem	= th_get_mem,
 
-.set_func =  th_set_func,
-.get_func =  th_get_func,
-.set_level =  th_set_level,
-.get_level =  th_get_level,
-.get_parm =  th_get_parm,
-.get_info =  th_get_info,
+	.set_func	= th_set_func,
+	.get_func	= th_get_func,
 
+	.set_level	= th_set_level,
+	.get_level	= th_get_level,
 
-.set_channel =  th_set_channel,
-.get_channel =  th_get_channel,
-.set_ant =  th_set_ant,
-.get_ant =  th_get_ant,
+	.get_parm	= th_get_parm,
+	.set_parm	= th_set_parm,
+
+	.get_info	= th_get_info,
+
+	.set_channel	= th_set_channel,
+	.get_channel	= th_get_channel,
+	.set_ant	= th_set_ant,
+	.get_ant	= th_get_ant,
+
+	.reset		= th_reset,
 };
 
-#define ACKBUF_LEN  64
 
 int thf7e_init(RIG *rig)
 {
@@ -267,119 +282,27 @@ int thf7e_open(RIG *rig)
 	 */
 	if (rig->state.itu_region != RIG_ITU_REGION1)
 		return -RIG_ECONF;
-	return RIG_OK;
+
+	return kenwood_open(rig);
 }
 
 /*
+ * th_get_vfo
+ * Assumes rig!=NULL
  */
 int
-thf7_set_vfo (RIG *rig, vfo_t vfo)
+thf7_get_vfo(RIG *rig, vfo_t *vfo)
 {
-    char vfobuf[16], ackbuf[ACKBUF_LEN];
-    int retval;
-    size_t ack_len;
+	char vfoch;
+	int retval;
 
-    rig_debug(RIG_DEBUG_TRACE, "%s: called\n", __func__);
+	rig_debug(RIG_DEBUG_TRACE, "%s: called\n", __func__);
 
-    /* 
-     * The band must be active before selecting VFO or MEM.
-     * The dilemma is whether MEM should be applied to Band A or Band B.
-     * Remember, not all bands have the same capability
-     * TODO: if (RIG_VFO_MEM) query current band with BC, then do appropriate VMC
-     */
-
-    if (vfo != RIG_VFO_MEM) {
-	switch (vfo) {
-        case RIG_VFO_A:
-        case RIG_VFO_VFO:
-        case RIG_VFO_MAIN:
-            sprintf(vfobuf, "BC 0%c", EOM_TH);
-            break;
-        case RIG_VFO_B:
-        case RIG_VFO_SUB:
-            sprintf(vfobuf, "BC 1%c", EOM_TH);
-            break;
-        default:
-            rig_debug(RIG_DEBUG_ERR, "%s: Unsupported BC VFO %d\n", __func__, vfo);
-            return -RIG_EVFO;
-	}
-
-	/* wait for response, otherwise next issued command may fail */
-    	ack_len = ACKBUF_LEN-1;
-    	retval = kenwood_transaction(rig, vfobuf, strlen(vfobuf), ackbuf, &ack_len);
-    	if (retval != RIG_OK)
-        	return retval;
-    }
-
-	switch (vfo) {
-        case RIG_VFO_VFO:
-        case RIG_VFO_MAIN:
-        case RIG_VFO_A:
-            sprintf(vfobuf, "VMC 0,0%c", EOM_TH);
-            break;
-        case RIG_VFO_SUB:
-        case RIG_VFO_B:
-            sprintf(vfobuf, "VMC 1,0%c", EOM_TH);
-            break;
-        case RIG_VFO_MEM:
-            sprintf(vfobuf, "VMC 0,1%c", EOM_TH);
-            break;
-        default:
-            rig_debug(RIG_DEBUG_ERR, "%s: Unsupported VFO %d\n", __func__, vfo);
-            return -RIG_EVFO;
-	}
-
-	/* wait for response, otherwise next issued command may fail */
-	ack_len = ACKBUF_LEN-1;
-	retval = kenwood_transaction(rig, vfobuf, strlen(vfobuf), ackbuf, &ack_len);
+	retval = th_get_vfo_char(rig, vfo, &vfoch);
 	if (retval != RIG_OK)
-        	return retval;
+		return retval;
 
-    return RIG_OK;
-}
-
-/*
- */
-int
-thf7_get_vfo (RIG *rig, vfo_t *vfo)
-{
-    char cmdbuf[16], ackbuf[ACKBUF_LEN],vfoc;
-    int retval;
-    size_t ack_len;
-
-    rig_debug(RIG_DEBUG_TRACE, "%s: called\n", __func__);
-
-    ack_len=ACKBUF_LEN;
-    /* XXX */
-    retval = kenwood_transaction (rig, "BC\r", 3, ackbuf, &ack_len);
-    if (retval != RIG_OK)
-        return retval;
-
-    if (ack_len < 4 ) {
-        rig_debug(RIG_DEBUG_ERR, "%s: Unexpected reply '%s'\n", __func__, ackbuf);
-        return -RIG_ERJCTED;
-    }
-
-    vfoc=ackbuf[3];
-    switch (vfoc) {
-        case '0': *vfo = RIG_VFO_A; break;
-        case '1': *vfo = RIG_VFO_B; break;
-        default:
-            rig_debug(RIG_DEBUG_ERR, "%s: Unexpected VFO value '%c'\n", __func__, ackbuf[3]);
-            return -RIG_EVFO;
-    }
-
-    sprintf(cmdbuf,"VMC %c%c",vfoc, EOM_TH);
-    ack_len=ACKBUF_LEN;
-    retval = kenwood_transaction (rig, cmdbuf, strlen(cmdbuf), ackbuf, &ack_len);
-    if (retval != RIG_OK)
-        return retval;
-
-    if (ack_len < 8 ) {
-        rig_debug(RIG_DEBUG_ERR, "%s: Unexpected reply '%s'\n", __func__, ackbuf);
-        return -RIG_ERJCTED;
-    }
-    switch(ackbuf[6]) {
+	switch (vfoch) {
 	case '0' :
 	case '3' :	/* Fine Step Enable */
 		break;
@@ -388,11 +311,10 @@ thf7_get_vfo (RIG *rig, vfo_t *vfo)
 	case '4' :	/* INFO */
 		*vfo = RIG_VFO_MEM;
 		break;
-        default:
-            rig_debug(RIG_DEBUG_ERR, "%s: Unexpected VFO value '%c'\n", __func__, ackbuf[6]);
-            return -RIG_EVFO;
-     }
+	default:
+		rig_debug(RIG_DEBUG_ERR, "%s: Unexpected VFO value '%c'\n", __func__, vfoch);
+		return -RIG_EVFO;
+	}
 
-     return RIG_OK;
+	return RIG_OK;
 }
-
