@@ -2,7 +2,7 @@
  *  Hamlib Kenwood backend - TS480 description
  *  Copyright (c) 2000-2004 by Stephane Fillod and Juergen Rinas
  *
- *	$Id: ts480.c,v 1.8 2008/10/26 13:48:17 y32kn Exp $
+ *	$Id: ts480.c,v 1.12 2009-02-03 22:56:06 azummo Exp $
  *
  *   This library is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU Library General Public License as
@@ -49,69 +49,12 @@
 static int
 kenwood_ts480_set_ptt (RIG * rig, vfo_t vfo, ptt_t ptt)
 {
-  char ackbuf[50];
-  size_t ack_len = 0;
-
   if (RIG_PTT_ON == ptt)
-    return kenwood_transaction (rig, "TX1;", 4, ackbuf, &ack_len);
-  return kenwood_transaction (rig, "RX;", 3, ackbuf, &ack_len);
+    return kenwood_simple_cmd(rig, "TX1");
+
+  return kenwood_simple_cmd(rig, "RX");
 }
 
-
-/*
- * kenwood_ts480_set_ant
- * Assumes rig!=NULL
- *
- * set the aerial/antenna  to use
- */
-static int
-kenwood_ts480_set_ant (RIG * rig, vfo_t vfo, ant_t ant)
-{
-  char ackbuf[50];
-  size_t ack_len = 0;
-
-  if (RIG_ANT_1 == ant)
-    return kenwood_transaction (rig, "AN1;", 4, ackbuf, &ack_len);
-  if (RIG_ANT_2 == ant)
-    return kenwood_transaction (rig, "AN2;", 4, ackbuf, &ack_len);
-  return -RIG_EINVAL;
-}
-
-
-/*
- * kenwood_ts480_get_ant
- * Assumes rig!=NULL
- *
- * get the aerial/antenna  in use
- */
-static int
-kenwood_ts480_get_ant (RIG * rig, vfo_t vfo, ant_t * ant)
-{
-  char ackbuf[50];
-  size_t ack_len = 50;
-  int retval;
-
-  retval = kenwood_transaction (rig, "AN;", 3, ackbuf, &ack_len);
-  if (RIG_OK != retval)
-    return retval;
-  if (4 != ack_len)
-    return -RIG_EPROTO;
-  switch (ackbuf[2])
-    {
-    case '1':
-      *ant = RIG_ANT_1;
-      break;
-    case '2':
-      *ant = RIG_ANT_2;
-      break;
-    default:
-      /* can only be a protocol error since the ts480 has only two antenna connectors */
-      return -RIG_EPROTO;
-    }
-  return RIG_OK;
-}
-
-
 /*
  * kenwood_ts480_get_info
  * Assumes rig!=NULL
@@ -124,7 +67,7 @@ kenwood_ts480_get_info (RIG * rig)
   int retval;
 
   firm_len = 50;
-  retval = kenwood_transaction (rig, "TY;", 3, firmbuf, &firm_len);
+  retval = kenwood_transaction (rig, "TY", 2, firmbuf, &firm_len);
   if (retval != RIG_OK)
     return NULL;
   if (firm_len != 6)
@@ -160,31 +103,29 @@ kenwood_ts480_get_info (RIG * rig)
 int
 kenwood_ts480_set_level (RIG * rig, vfo_t vfo, setting_t level, value_t val)
 {
-  char levelbuf[16], ackbuf[50];
-  int level_len, retval;
-  size_t ack_len;
+  char levelbuf[16];
   int kenwood_val;
 
   switch (level)
     {
     case RIG_LEVEL_RFPOWER:
       kenwood_val = val.f * 100;	/* level for TS480SAT is from 0.. 100W in SSB */
-      level_len = sprintf (levelbuf, "PC%03d;", kenwood_val);
+      sprintf (levelbuf, "PC%03d", kenwood_val);
       break;
 
     case RIG_LEVEL_AF:
       kenwood_val = val.f * 255;	/* possible values for TS480 are 000.. 255 */
-      level_len = sprintf (levelbuf, "AG0%03d;", kenwood_val);
+      sprintf (levelbuf, "AG0%03d", kenwood_val);
       break;
 
     case RIG_LEVEL_RF:
       kenwood_val = val.f * 100;	/* possible values for TS480 are 000.. 100 */
-      level_len = sprintf (levelbuf, "RG%03d;", kenwood_val);
+      sprintf (levelbuf, "RG%03d", kenwood_val);
       break;
 
     case RIG_LEVEL_SQL:
       kenwood_val = val.f * 255;	/* possible values for TS480 are 000.. 255 */
-      level_len = sprintf (levelbuf, "SQ0%03d;", kenwood_val);
+      sprintf (levelbuf, "SQ0%03d", kenwood_val);
       break;
 
     case RIG_LEVEL_AGC:	/* possible values for TS480 000(=off), 001(=fast), 002(=slow) */
@@ -204,7 +145,7 @@ kenwood_ts480_set_level (RIG * rig, vfo_t vfo, setting_t level, value_t val)
 	  rig_debug (RIG_DEBUG_ERR, "Unsupported agc value");
 	  return -RIG_EINVAL;
 	};
-      level_len = sprintf (levelbuf, "GT%03d;", kenwood_val);
+      sprintf (levelbuf, "GT%03d", kenwood_val);
       break;
 
     default:
@@ -212,12 +153,7 @@ kenwood_ts480_set_level (RIG * rig, vfo_t vfo, setting_t level, value_t val)
       return -RIG_EINVAL;
     }
 
-  ack_len = 0;
-  retval = kenwood_transaction (rig, levelbuf, level_len, ackbuf, &ack_len);
-  if (retval != RIG_OK)
-    return retval;
-
-  return RIG_OK;
+  return kenwood_simple_cmd(rig, levelbuf);
 }
 
 
@@ -236,7 +172,7 @@ kenwood_ts480_get_level (RIG * rig, vfo_t vfo, setting_t level, value_t * val)
   switch (level)
     {
     case RIG_LEVEL_RFPOWER:
-      retval = kenwood_transaction (rig, "PC;", 3, ackbuf, &ack_len);
+      retval = kenwood_transaction (rig, "PC", 2, ackbuf, &ack_len);
       if (RIG_OK != retval)
 	return retval;
       if (6 != ack_len)
@@ -247,7 +183,7 @@ kenwood_ts480_get_level (RIG * rig, vfo_t vfo, setting_t level, value_t * val)
       return RIG_OK;
 
     case RIG_LEVEL_AF:
-      retval = kenwood_transaction (rig, "AG0;", 4, ackbuf, &ack_len);
+      retval = kenwood_transaction (rig, "AG0", 3, ackbuf, &ack_len);
       if (RIG_OK != retval)
 	return retval;
       if (7 != ack_len)
@@ -258,7 +194,7 @@ kenwood_ts480_get_level (RIG * rig, vfo_t vfo, setting_t level, value_t * val)
       return RIG_OK;
 
     case RIG_LEVEL_RF:
-      retval = kenwood_transaction (rig, "RG;", 3, ackbuf, &ack_len);
+      retval = kenwood_transaction (rig, "RG", 2, ackbuf, &ack_len);
       if (RIG_OK != retval)
 	return retval;
       if (6 != ack_len)
@@ -269,7 +205,7 @@ kenwood_ts480_get_level (RIG * rig, vfo_t vfo, setting_t level, value_t * val)
       return RIG_OK;
 
     case RIG_LEVEL_SQL:
-      retval = kenwood_transaction (rig, "SQ0;", 4, ackbuf, &ack_len);
+      retval = kenwood_transaction (rig, "SQ0", 3, ackbuf, &ack_len);
       if (RIG_OK != retval)
 	return retval;
       if (7 != ack_len)
@@ -280,7 +216,7 @@ kenwood_ts480_get_level (RIG * rig, vfo_t vfo, setting_t level, value_t * val)
       return RIG_OK;
 
     case RIG_LEVEL_AGC:
-      retval = kenwood_transaction (rig, "GT;", 3, ackbuf, &ack_len);
+      retval = kenwood_transaction (rig, "GT", 2, ackbuf, &ack_len);
       if (RIG_OK != retval)
 	return retval;
       if (6 != ack_len)
@@ -324,47 +260,7 @@ kenwood_ts480_get_level (RIG * rig, vfo_t vfo, setting_t level, value_t * val)
   return RIG_OK;		/* never reached */
 }
 
-
-int
-kenwood_ts480_set_func (RIG * rig, vfo_t vfo, setting_t func, int status)
-{
-  char fctbuf[16], ackbuf[50];
-  int fct_len;
-  size_t ack_len;
-
-  ack_len = 0;
-  switch (func)
-    {
-    case RIG_FUNC_NB:
-      fct_len = sprintf (fctbuf, "NB%c;", status == 0 ? '0' : '1');
-      return kenwood_transaction (rig, fctbuf, fct_len, ackbuf, &ack_len);
-
-    case RIG_FUNC_COMP:
-      fct_len = sprintf (fctbuf, "PR%c;", status == 0 ? '0' : '1');
-      return kenwood_transaction (rig, fctbuf, fct_len, ackbuf, &ack_len);
-
-    case RIG_FUNC_VOX:
-      fct_len = sprintf (fctbuf, "VX%c;", status == 0 ? '0' : '1');
-      return kenwood_transaction (rig, fctbuf, fct_len, ackbuf, &ack_len);
-
-    case RIG_FUNC_NR:
-      fct_len = sprintf (fctbuf, "NR%c;", status == 0 ? '0' : '1');
-      return kenwood_transaction (rig, fctbuf, fct_len, ackbuf, &ack_len);
-
-    case RIG_FUNC_BC:
-      fct_len = sprintf (fctbuf, "BC%c;", status == 0 ? '0' : '1');
-      return kenwood_transaction (rig, fctbuf, fct_len, ackbuf, &ack_len);
-
-    default:
-      rig_debug (RIG_DEBUG_ERR, "Unsupported set_func %#x", func);
-      return -RIG_EINVAL;
-    }
-
-  return RIG_OK;
-}
-
-
-static const struct kenwood_priv_caps ts480_priv_caps = {
+static struct kenwood_priv_caps ts480_priv_caps = {
   .cmdtrm = EOM_KEN,
 };
 
@@ -487,6 +383,8 @@ const struct rig_caps ts480_caps = {
                 RIG_FLT_END,
         },
   .priv = (void *) &ts480_priv_caps,
+  .rig_init = kenwood_init,
+  .rig_cleanup = kenwood_cleanup,
   .set_freq = kenwood_set_freq,
   .get_freq = kenwood_get_freq,
   .set_rit = kenwood_set_rit,	/*  FIXME should this switch to rit mode or just set the frequency? */
@@ -496,7 +394,7 @@ const struct rig_caps ts480_caps = {
   .set_mode = kenwood_set_mode,
   .get_mode = kenwood_get_mode,
   .set_vfo = kenwood_set_vfo,
-  .get_vfo = kenwood_get_vfo,
+  .get_vfo = kenwood_get_vfo_if,
   .get_ptt = kenwood_get_ptt,
   .set_ptt = kenwood_ts480_set_ptt,
   .get_dcd = kenwood_get_dcd,
@@ -504,8 +402,8 @@ const struct rig_caps ts480_caps = {
   .get_powerstat = kenwood_get_powerstat,
   .get_info = kenwood_ts480_get_info,
   .reset = kenwood_reset,
-  .set_ant = kenwood_ts480_set_ant,
-  .get_ant = kenwood_ts480_get_ant,
+  .set_ant = kenwood_set_ant,
+  .get_ant = kenwood_get_ant,
   .scan = kenwood_scan,		/* not working, invalid arguments using rigctl; kenwood_scan does only support on/off and not tone and CTCSS scan */
   .has_set_level = TS480_LEVEL_ALL,
   .has_get_level = TS480_LEVEL_ALL,
@@ -513,7 +411,7 @@ const struct rig_caps ts480_caps = {
   .get_level = kenwood_ts480_get_level,
   .has_get_func = TS480_FUNC_ALL,
   .has_set_func = TS480_FUNC_ALL,
-  .set_func = kenwood_ts480_set_func,
+  .set_func = kenwood_set_func,
   .get_func = kenwood_get_func,
 };
 
