@@ -1,8 +1,7 @@
 /*
  *  Hamlib Kenwood backend - TS690 description
- *  Copyright (c) 2000-2004 by Stephane Fillod
+ *  Copyright (c) 2000-2010 by Stephane Fillod
  *
- *	$Id: ts690.c,v 1.7 2009-02-03 22:42:44 azummo Exp $
  *
  *   This library is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU Library General Public License as
@@ -68,7 +67,6 @@ static struct kenwood_priv_caps  ts690_priv_caps  = {
  * 	http://www.qsl.net/sm7vhs/radio/kenwood/ts690/specs.htm
  *
  * TODO: protocol to be checked with manual!
- * 	- get_channel/set_channel: MR/MW
  * 	- how to set_split in vfo mode?
  * 	- ...
  */
@@ -76,9 +74,9 @@ const struct rig_caps ts690s_caps = {
 .rig_model =  RIG_MODEL_TS690S,
 .model_name = "TS-690S",
 .mfg_name =  "Kenwood",
-.version =  BACKEND_VER,
+.version =  BACKEND_VER ".1",
 .copyright =  "LGPL",
-.status =  RIG_STATUS_ALPHA,
+.status =  RIG_STATUS_BETA,
 .rig_type =  RIG_TYPE_TRANSCEIVER,
 .ptt_type =  RIG_PTT_RIG,
 .dcd_type =  RIG_DCD_RIG,
@@ -174,9 +172,11 @@ const struct rig_caps ts690s_caps = {
 .set_xit =  kenwood_set_xit,
 .get_xit =  kenwood_get_xit,
 .set_mode =  kenwood_set_mode,
-.get_mode =  kenwood_get_mode,
+.get_mode =  kenwood_get_mode_if, 
 .set_vfo =  kenwood_set_vfo,
 .get_vfo =  kenwood_get_vfo_if,
+.set_split_vfo =  kenwood_set_split_vfo,
+.get_split_vfo =  kenwood_get_split_vfo_if,
 .get_ptt =  kenwood_get_ptt,
 .set_ptt =  kenwood_set_ptt,
 .get_dcd =  kenwood_get_dcd,
@@ -184,12 +184,19 @@ const struct rig_caps ts690s_caps = {
 .get_func =  kenwood_get_func,
 .set_level =  kenwood_set_level,
 .get_level =  kenwood_get_level,
+.set_ext_parm = kenwood_set_ext_parm,
+.get_ext_parm = kenwood_get_ext_parm,
 .vfo_op =  kenwood_vfo_op,
 .set_mem =  kenwood_set_mem,
 .get_mem =  kenwood_get_mem_if,
 .set_trn =  kenwood_set_trn,
 .get_trn =  kenwood_get_trn,
+.set_powerstat = kenwood_set_powerstat,
+.get_powerstat = kenwood_get_powerstat,
+.reset = kenwood_reset,
 .scan =  kenwood_scan,
+.get_channel = kenwood_get_channel,
+.set_channel = kenwood_set_channel,
 
 };
 
