@@ -1,7 +1,7 @@
 /*
  * Hamlib backend library for the DCU rotor command set.
  *
- * rotorez.h - (C) Nate Bargmann 2003, 2009 (n0nb at arrl.net)
+ * rotorez.h - (C) Nate Bargmann 2003,2009,2010 (n0nb at arrl.net)
  *
  * This shared library provides an API for communicating
  * via serial interface to a Hy-Gain or Yaesu rotor using
@@ -9,9 +9,6 @@
  * supports the Hy-Gain DCU-1.
  *
  *
- *    $Id: rotorez.h,v 1.8 2009-01-25 16:14:39 fillods Exp $
- *
- *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU Lesser General Public
  *  License as published by the Free Software Foundation; either
@@ -38,6 +35,7 @@
 extern const struct rot_caps rotorez_rot_caps;
 extern const struct rot_caps rotorcard_rot_caps;
 extern const struct rot_caps dcu_rot_caps;
+extern const struct rot_caps erc_rot_caps;
 
 /*
  * Tokens used by rotorez_rot_set_conf and the 'C' command in rotctl
@@ -59,8 +57,10 @@ static int rotorez_rot_cleanup(ROT *rot);
 
 static int rotorez_rot_set_position(ROT *rot, azimuth_t azimuth, elevation_t elevation);
 static int rotorez_rot_get_position(ROT *rot, azimuth_t *azimuth, elevation_t *elevation);
+static int erc_rot_get_position(ROT *rot, azimuth_t *azimuth, elevation_t *elevation);
 
 static int rotorez_rot_stop(ROT *rot);
+static int dcu1_rot_stop(ROT *rot);
 
 static int rotorez_rot_set_conf(ROT *rot, token_t token, const char *val);
 
