@@ -1,27 +1,25 @@
 /*
- * rotctl.c - (C) Stephane Fillod 2000-2009
+ * rotctl.c - (C) Stephane Fillod 2000-2010
  *
  * This program test/control a rotator using Hamlib.
- * It takes commands in interactive mode as well as 
+ * It takes commands in interactive mode as well as
  * from command line options.
  *
- *	$Id: rotctl.c,v 1.14 2009-02-17 08:03:22 fillods Exp $  
- *
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; either version 2
  * of the License, or (at your option) any later version.
- * 
+ *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
- * 
+ *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
- * 
+ *
  */
 
 #ifdef HAVE_CONFIG_H
@@ -41,18 +39,18 @@
 
 #include "rotctl_parse.h"
 
-/* 
+/*
  * Prototypes
  */
 void usage();
 
 /*
- * Reminder: when adding long options, 
+ * Reminder: when adding long options,
  * keep up to date SHORT_OPTIONS, usage()'s output and man page. thanks.
  * NB: do NOT use -W since it's reserved by POSIX.
  * TODO: add an option to read from a file
  */
-#define SHORT_OPTIONS "m:r:s:C:t:LvhVl"
+#define SHORT_OPTIONS "m:r:s:C:t:LvhVlu"
 static struct option long_options[] =
 {
 	{"model",    1, 0, 'm'},
@@ -62,6 +60,7 @@ static struct option long_options[] =
 	{"list",     0, 0, 'l'},
 	{"set-conf", 1, 0, 'C'},
 	{"show-conf",0, 0, 'L'},
+	{"dump-caps",0, 0, 'u'},
 	{"verbose",  0, 0, 'v'},
 	{"help",     0, 0, 'h'},
 	{"version",  0, 0, 'V'},
@@ -77,7 +76,7 @@ int opt_end= 0 ;        /* only used by rotctld */
 char send_cmd_term = '\r';     /* send_cmd termination char */
 
 int main (int argc, char *argv[])
-{ 
+{
 	ROT *my_rot;		/* handle to rot (instance) */
 	rot_model_t my_model = ROT_MODEL_DUMMY;
 
@@ -85,6 +84,7 @@ int main (int argc, char *argv[])
 
 	int verbose = 0;
 	int show_conf = 0;
+	int dump_caps_opt = 0;
 	const char *rot_file=NULL;
 	int serial_rate = 0;
 	char conf_parms[MAXCONFLEN] = "";
@@ -154,20 +154,23 @@ int main (int argc, char *argv[])
 			case 'l':
 					list_models();
 					exit(0);
+			case 'u':
+					dump_caps_opt++;
+					break;
 			default:
 					usage();	/* unknown option? */
 					exit(1);
 		}
 	}
-	if (verbose < 2)
-		rig_set_debug(RIG_DEBUG_WARN);
+
+	rig_set_debug(verbose);
 
 	rig_debug(RIG_DEBUG_VERBOSE, "rotctl, %s\n", hamlib_version);
 	rig_debug(RIG_DEBUG_VERBOSE, "Report bugs to "
 			"<hamlib-developer@lists.sourceforge.net>\n\n");
 
 	/*
-	 * at least one command on command line, 
+	 * at least one command on command line,
 	 * disable interactive mode
 	 */
 	if (optind < argc)
@@ -176,7 +179,7 @@ int main (int argc, char *argv[])
   	my_rot = rot_init(my_model);
 
 	if (!my_rot) {
-		fprintf(stderr, "Unknown rot num %d, or initialization error.\n", 
+		fprintf(stderr, "Unknown rot num %d, or initialization error.\n",
 						my_model);
 		fprintf(stderr, "Please check with --list option.\n");
 		exit(2);
@@ -202,6 +205,16 @@ int main (int argc, char *argv[])
 		rot_token_foreach(my_rot, print_conf_list, (rig_ptr_t)my_rot);
 	}
 
+    /*
+     * print out capabilities, and exists immediately
+     * We may be interested only in caps, and rig_open may fail.
+     */
+    if (dump_caps_opt) {
+        dumpcaps_rot(my_rot, stdout);
+        rot_cleanup(my_rot); /* if you care about memory */
+        exit(0);
+    }
+
 	retcode = rot_open(my_rot);
 	if (retcode != RIG_OK) {
 	  	fprintf(stderr,"rot_open: error = %s \n", rigerror(retcode));
@@ -240,6 +253,7 @@ void usage()
 	"  -C, --set-conf=PARM=VAL    set config parameters\n"
 	"  -L, --show-conf            list all config parameters\n"
 	"  -l, --list                 list all model numbers and exit\n"
+    "  -u, --dump-caps            dump capabilities and exit\n"
 	"  -v, --verbose              set verbose mode, cumulative\n"
 	"  -h, --help                 display this help and exit\n"
 	"  -V, --version              output version information and exit\n\n"
