@@ -55,7 +55,7 @@ const struct rig_caps ft2000_caps = {
     .mfg_name =           "Yaesu",
     .version =            NEWCAT_VER,
     .copyright =          "LGPL",
-    .status =             RIG_STATUS_UNTESTED,
+    .status =             RIG_STATUS_STABLE,
     .rig_type =           RIG_TYPE_TRANSCEIVER,
     .ptt_type =           RIG_PTT_RIG,
     .dcd_type =           RIG_DCD_NONE,
